"""Constants for versasense."""
KEY_CONSUMER = "consumer"
KEY_IDENTIFIER = "identifier"
KEY_MEASUREMENT = "measurement"
KEY_PARENT_MAC = "parent_mac"
KEY_PARENT_NAME = "parent_name"
KEY_UNIT = "unit"
PERIPHERAL_CLASS_SENSOR = "sensor"
PERIPHERAL_CLASS_SENSOR_ACTUATOR = "sensor-actuator"
PERIPHERAL_STATE_OFF = "OFF"
PERIPHERAL_STATE_ON = "ON"
