"""Automatically generated by hassfest.

To update, run python3 -m script.hassfest
"""

# fmt: off

ZEROCONF = {
    "_api._udp.local.": [
        "guardian"
    ],
    "_axis-video._tcp.local.": [
        "axis",
        "doorbird"
    ],
    "_daap._tcp.local.": [
        "forked_daapd"
    ],
    "_dkapi._tcp.local.": [
        "daikin"
    ],
    "_elg._tcp.local.": [
        "elgato"
    ],
    "_esphomelib._tcp.local.": [
        "esphome"
    ],
    "_googlecast._tcp.local.": [
        "cast"
    ],
    "_hap._tcp.local.": [
        "homekit_controller"
    ],
    "_ipp._tcp.local.": [
        "ipp"
    ],
    "_ipps._tcp.local.": [
        "ipp"
    ],
    "_miio._udp.local.": [
        "xiaomi_miio"
    ],
    "_printer._tcp.local.": [
        "brother"
    ],
    "_spotify-connect._tcp.local.": [
        "spotify"
    ],
    "_viziocast._tcp.local.": [
        "vizio"
    ],
    "_wled._tcp.local.": [
        "wled"
    ]
}

HOMEKIT = {
    "819LMB": "myq",
    "AC02": "tado",
    "BSB002": "hue",
    "Healty Home Coach": "netatmo",
    "LIFX": "lifx",
    "Netatmo Relay": "netatmo",
    "PowerView": "hunterdouglas_powerview",
    "Presence": "netatmo",
    "Rachio": "rachio",
    "TRADFRI": "tradfri",
    "Welcome": "netatmo",
    "Wemo": "wemo",
    "tado": "tado"
}
