"""Constants for Bravia TV integration."""
ATTR_CID = "cid"
ATTR_MAC = "macAddr"
ATTR_MANUFACTURER = "Sony"
ATTR_MODEL = "model"

CONF_IGNORED_SOURCES = "ignored_sources"

BRAVIARC = "braviarc"
BRAVIA_CONFIG_FILE = "bravia.conf"
CLIENTID_PREFIX = "HomeAssistant"
DEFAULT_NAME = f"{ATTR_MANUFACTURER} Bravia TV"
DOMAIN = "braviatv"
NICKNAME = "Home Assistant"
UNDO_UPDATE_LISTENER = "undo_update_listener"
