"""Constants for the Twente Milieu integration."""

DOMAIN = "twentemilieu"

DATA_UPDATE = "twentemilieu_update"

CONF_POST_CODE = "post_code"
CONF_HOUSE_NUMBER = "house_number"
CONF_HOUSE_LETTER = "house_letter"
